% script file splines2a.m
% bi-cubic spline interpolation in 2D
% the same as splines2 only now 
% use different ordering to obtain better
% pattern (u and Du are neighbouring rows)
% quite unefficient, but hopefully a bit understandable version ;)
% 18.04.05 by vita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all;

% generate the data set
n = 10;

x = linspace(0,1,n)';
y = linspace(0,1,n)';

h=x(2)-x(1);

[X,Y] = ndgrid(x,y);

% load the rhs from the data file
F = load('experimental_result.dat');

% derivative values at the boundary points
% boundary consists of 4 parts

% y=0
bdry1_x = x; bdry1_y = zeros(n,1);
% values of normal derivative at the boundary 1
dF_bdry1 = zeros(n,1);


% x=1
bdry2_x = ones(n,1); bdry2_y = y;
dF_bdry2 = zeros(n,1);

% y=1
bdry3_x = x; bdry3_y = ones(n,1);
dF_bdry3 = zeros(n,1);

% x=0
bdry4_x = zeros(n,1); bdry4_y = y;
dF_bdry4 = zeros(n,1);

% contruct the rhs vector

RHS = build_spline_col(n,F,dF_bdry1,dF_bdry2,dF_bdry3,dF_bdry4);


% build the matrix
% initialise the memory
% dimensionality of the problem:
% all inner points once + boundary points once more
dim = n*n + 4*n;
A = sparse(dim,dim);

% first n columns of A correspond to 
% basis functions 
% j=0
% psi_{:,0}
for i=1:n
  % compute the operator F psi
  % psi at the grid points
  psi = sparse(basis((X-x(i))./h).*basis((Y-(-h))./h));
  
  % normal derivatives at the boundaries
  dF_psi1 = -1/h*basis((bdry1_x-x(i))./h).*Dbasis((bdry1_y-(-h))./h);
  dF_psi1 = sparse(dF_psi1);
  
  dF_psi2 = 1/h*Dbasis((bdry2_x-x(i))./h).*basis((bdry2_y-(-h))./h);
  dF_psi2 = sparse(dF_psi2);

  dF_psi3 = 1/h*basis((bdry3_x-x(i))./h).*Dbasis((bdry3_y-(-h))./h);
  dF_psi3 = sparse(dF_psi3);
  
  dF_psi4 = -1/h*Dbasis((bdry4_x-x(i))./h).*basis((bdry4_y-(-h))./h);
  dF_psi4 = sparse(dF_psi4);
  
  col = build_spline_col(n,psi,dF_psi1,dF_psi2,dF_psi3,dF_psi4);
           
  A(:,i) = col;
end

% column counter
ctr = n;  

% points with j=1:n
for j=1:n
  % left boundary point treated separately
  
  % compute the operator F psi
  % psi at the grid points
  psi = sparse(basis((X-(-h))./h).*basis((Y-y(j))./h));
  
  % normal derivatives at the boundaries
  dF_psi1 = -1/h*basis((bdry1_x-(-h))./h).*Dbasis((bdry1_y-y(j))./h);
  dF_psi1 = sparse(dF_psi1);
  
  dF_psi2 = 1/h*Dbasis((bdry2_x-(-h))./h).*basis((bdry2_y-y(j))./h);
  dF_psi2 = sparse(dF_psi2);

  dF_psi3 = 1/h*basis((bdry3_x-(-h))./h).*Dbasis((bdry3_y-y(j))./h);
  dF_psi3 = sparse(dF_psi3);
  
  dF_psi4 = -1/h*Dbasis((bdry4_x-(-h))./h).*basis((bdry4_y-y(j))./h);
  dF_psi4 = sparse(dF_psi4);

  col = build_spline_col(n,psi,dF_psi1,dF_psi2,dF_psi3,dF_psi4);
         
  ctr = ctr+1;  
  A(:,ctr) = col;
 
  
  % the inner basis functions
  for i=1:n
    % compute the operator F psi
    % psi at the grid points
    psi = sparse(basis((X-x(i))./h).*basis((Y-y(j))./h));
  
    % normal derivatives at the boundaries
    dF_psi1 = -1/h*basis((bdry1_x-x(i))./h).*Dbasis((bdry1_y-y(j))./h);
    dF_psi1 = sparse(dF_psi1);
  
    dF_psi2 = 1/h*Dbasis((bdry2_x-x(i))./h).*basis((bdry2_y-y(j))./h);
    dF_psi2 = sparse(dF_psi2);

    dF_psi3 = 1/h*basis((bdry3_x-x(i))./h).*Dbasis((bdry3_y-y(j))./h);
    dF_psi3 = sparse(dF_psi3);
  
    dF_psi4 = -1/h*Dbasis((bdry4_x-x(i))./h).*basis((bdry4_y-y(j))./h);
    dF_psi4 = sparse(dF_psi4);

    col = build_spline_col(n,psi,dF_psi1,dF_psi2,dF_psi3,dF_psi4);
       
    ctr = ctr+1;  
    A(:,ctr) = col;    
            
  end
  
  % right boundary treated separately
  % compute the operator F psi
  % psi at the grid points
  psi = sparse(basis((X-(1+h))./h).*basis((Y-y(j))./h));
  
  % normal derivatives at the boundaries
  dF_psi1 = -1/h*basis((bdry1_x-(1+h))./h).*Dbasis((bdry1_y-y(j))./h);
  dF_psi1 = sparse(dF_psi1);
  
  dF_psi2 = 1/h*Dbasis((bdry2_x-(1+h))./h).*basis((bdry2_y-y(j))./h);
  dF_psi2 = sparse(dF_psi2);

  dF_psi3 = 1/h*basis((bdry3_x-(1+h))./h).*Dbasis((bdry3_y-y(j))./h);
  dF_psi3 = sparse(dF_psi3);
  
  dF_psi4 = -1/h*Dbasis((bdry4_x-(1+h))./h).*basis((bdry4_y-y(j))./h);
  dF_psi4 = sparse(dF_psi4);
  
  col = build_spline_col(n,psi,dF_psi1,dF_psi2,dF_psi3,dF_psi4);
         
  ctr = ctr+1;  
  A(:,ctr) = col;  
                
end

% last n columns of A correspond to 
% basis functions 
% j=n+1
% psi_{:,n+1}
for i=1:n
  % compute the operator F psi
  % psi at the grid points
  psi = sparse(basis((X-x(i))./h).*basis((Y-(1+h))./h));
  
  % normal derivatives at the boundaries
  dF_psi1 = -1/h*basis((bdry1_x-x(i))./h).*Dbasis((bdry1_y-(1+h))./h);
  dF_psi1 = sparse(dF_psi1);
  
  dF_psi2 = 1/h*Dbasis((bdry2_x-x(i))./h).*basis((bdry2_y-(1+h))./h);
  dF_psi2 = sparse(dF_psi2);

  dF_psi3 = 1/h*basis((bdry3_x-x(i))./h).*Dbasis((bdry3_y-(1+h))./h);
  dF_psi3 = sparse(dF_psi3);
  
  dF_psi4 = -1/h*Dbasis((bdry4_x-x(i))./h).*basis((bdry4_y-(1+h))./h);
  dF_psi4 = sparse(dF_psi4);

  col = build_spline_col(n,psi,dF_psi1,dF_psi2,dF_psi3,dF_psi4);
           
  ctr = ctr+1;  
  A(:,ctr) = col;  
  
end

U = A\RHS;

% visualise the interpolated function
% on much finer mesh
xx = linspace(0,1,10*n);
yy = linspace(0,1,10*n);

[XX,YY] = ndgrid(xx,yy);

% sum the interpolants
P = zeros(10*n,10*n);

% j=-1;
for i=1:n
  P = P+U(i)*basis((XX-x(i))./h).*basis((YY-(-h))./h);
end
 
ctr = n;
for j=1:n
  ctr = ctr+1;
  P = P+U(ctr)*basis((XX-(-h))./h).*basis((YY-y(j))./h);
  
  for i=1:n
    ctr = ctr+1;
    P = P+U(ctr)*basis((XX-x(i))./h).*basis((YY-y(j))./h);
  end
  
  ctr = ctr+1;
  P = P+U(ctr)*basis((XX-(1+h))./h).*basis((YY-y(j))./h);

end

% j=n+1;
for i=1:n
  ctr = ctr+1; 
  P = P+U(ctr)*basis((XX-x(i))./h).*basis((YY-(1+h))./h);
end

figure(1)
surf(XX,YY,P);
hold on

N = n*n;
plot3(reshape(X,N,1),reshape(Y,N,1),reshape(F,N,1),'r*');  

xlabel('x');
ylabel('y');

shading flat